/*
 * Created on Jun 23, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import java.util.Vector;  
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.* ;
import edu.ncsu.csc.mas.ontology.exceptions.*;
import edu.ncsu.csc.mas.utils.Constants;
import edu.ncsu.csc.mas.utils.OwlpGenerator;

/**
 * Represents the OWLP ontology message. the design goal is to have this class
 * represent strictly ontological attributes of a message. 
 * <p>
 * Each Message has a sender, a receiver, a message type, and parameters that represent 
 * parameters that depend on the message type.
 * <p>
 * This class is anticipated to have several subclasses representing various kinds of messages
 * e.g. protocol messages, control messages, and infrastructure messages for various
 * implementations. For now,
 * only a <code>JJMessage</code> (for Jess/J2EE enactment of OWLP) is in use.
 * 
 * @author Nirmit Desai
 *
 */
public class Message extends OwlpObject {
    
	/**
     * The sender of this message
     */
    protected Role sender ;
    
    /**
     * The receiver of this message
     */
    protected Role receiver ;
    
    /**
     * The parameters of this message. Variable in number.
     */
    protected Vector parameters ;
    
    /**
     * Message type
     */
    protected String type ;
    
   
    

    /**
     * Return a fresh <code>Message</code> object
     *
     */
    public Message() {
        
    }
    
    public Message(String t, Role s, Role r, Vector p) {
        if(t!=null)
            this.type = new String(t) ;
        if(s!=null)
            this.sender = new Role(s) ;
        if(r!=null)
            this.receiver = new Role(r) ;
        this.parameters = new Vector(p) ;
        
    }
    
    /**
     * Constructs a Message object from a given Messagestub object
     * 
     * @param mStub The object to use create the fresh object
     * @throws MalformedMessageException
     */
    public Message(Message mStub)
    throws MalformedMessageException{
        
        MalformedMessageException mme = new MalformedMessageException();
        
        if(mStub.getSender() == null) throw mme;
        this.sender = new Role(mStub.getSender()) ;
        
        if(mStub.getReceiver() == null) throw mme;
        this.receiver = new Role(mStub.getReceiver()) ;
        
        if(mStub.getParameters() != null)
            this.parameters = (Vector) mStub.getParameters().clone() ;
        
        if(mStub.getType() == null) throw mme ;
        this.type = new String(mStub.getType()) ;
        
    }
    
    /**
     * Compare two messages
     * 
     * @param ms the message to compare with
     * @return true if this message is equal to ms
     */
    public boolean equals(Message ms)
    {
        return false ;
    }
    
    /**
     * Return a fresh <code>Message</code> object given its JENA equivalent
     *
     */
    public Message(OntResource msg)
    throws MalformedConditionException
    {
    	StmtIterator sIter ;
       //Find out the type, sender and receiver of this message
        if(Constants.DEBUG) System.out.println("Message - " + msg.getLocalName()) ;
        this.type = new String(msg.getLocalName()) ;
        //list all statements in which this message is the subject
        for(sIter=msg.listProperties(); sIter.hasNext(); ) {
            //get the next statement in which this message is the subject
            Statement msgSt = sIter.nextStatement();
            //get the object of the statement
            RDFNode objRDFNode = msgSt.getObject() ;
            OntResource msgObj ;
            //try and convert this object into an OntResource
            if(objRDFNode.canAs(OntResource.class)) {
                msgObj = (OntResource)objRDFNode.as(OntResource.class) ;
            } else 
                continue ;
            //Now see what property relates the message to the object
            if(msgSt.getPredicate().getLocalName().equals(Constants.MESSAGE_SENDER_PROPERTY_NAME)) {
                //the property relates this message to its sender
                this.setSender(new Role(msgObj)) ;
                if(Constants.DEBUG) System.out.println( Constants.MESSAGE_SENDER_PROPERTY_NAME+ " - " + msgObj.getLocalName()) ;
            } else if(msgSt.getPredicate().getLocalName().equals(Constants.MESSAGE_RECEIVER_PROPERTY_NAME)) {
                //the property relates this message to its receiver
                this.setReceiver(new Role(msgObj)) ;
                if(Constants.DEBUG) System.out.println( Constants.MESSAGE_RECEIVER_PROPERTY_NAME+ " - " + msgObj.getLocalName()) ;
            } else if(msgSt.getPredicate().getLocalName().equals(Constants.MESSAGE_PARAMETER_PROPERTY_NAME)) {
                RDFNode msgObjRDFNode = msgSt.getObject() ;
                if(msgObjRDFNode.canAs(OntResource.class)) {
                    msgObj = (OntResource)msgObjRDFNode.as(OntResource.class);
                } else continue ;
                //one of the parameters of this message
                if(this.parameters == null)
                    this.parameters = new Vector() ;
                if(msgObj.getRDFType().getLocalName().equals("BooleanSlot"))
                	this.parameters.add(new BooleanSlot(msgObj)) ;
                else if(msgObj.getRDFType().getLocalName().equals("FloatSlot"))
                	this.parameters.add(new FloatSlot(msgObj)) ;
                else if(msgObj.getRDFType().getLocalName().equals("IntegerSlot"))
                	this.parameters.add(new IntegerSlot(msgObj)) ;
                else if(msgObj.getRDFType().getLocalName().equals("StringSlot"))
                	this.parameters.add(new StringSlot(msgObj)) ;
                if(Constants.DEBUG) System.out.println( Constants.MESSAGE_PARAMETER_PROPERTY_NAME + " - " + msgObj.getLocalName()) ;
	        }
        }
        
    }
    
    
    public String toString() {
        String retString = new String("(") ;
        retString += this.type + "_msg ";
        if(this.sender != null)
//            retString += " sndr:" + this.sender.toString() + " ";
        if(this.receiver!=null)
            retString += " rcvr:" + this.receiver.toString() + " ";
/*        if(this.parameters!= null) {
            for(int i=0; i<this.parameters.size(); i++) {
                retString += " ?" + ((Slot)this.parameters.elementAt(i)).toString() ;
            }
        }
*/        
        retString += ")" ;
        return retString   ;
    }
  
    /**
     * @author pchakra 
     *
     */

    
    public String toSendMsgJessString(int num)
    {
    	String retString = "";
    	
    	retString+="(bind ?" + this.type+"-var (new " + this.type;
    	if(this.parameters!=null)
    	for(int i=0;i<this.parameters.size();i++)
    	{
    		String slot = ((Slot)this.parameters.elementAt(i)).toJessString();
            retString +=  " ?" + slot + "-var" +num ;
           
    	}
    		
    	retString+=" ?t" +num+ "))";
    	retString+="\n\t(definstance " +this.type+ "MsgProp ?" + this.type+ "-var static)";
    	retString+="\n\t(store OUTPUT ?" + this.type+ "-var))";
    	
    	
    	
    	return retString;
    }
    public String toRecvMsgJessString(int num)
    {
    	String retString = "";
    	
    	retString+="(bind ?" + this.type+"-var (new " + this.type;
    	if(this.parameters!=null)
    	for(int i=0;i<this.parameters.size();i++)
    	{
    		String slot = ((Slot)this.parameters.elementAt(i)).toJessString();
            retString +=  " ?" + slot + "-var" + num;
           
    	}
    		
    	retString+=" ?t" + num +"))";
    	retString+="\n\t(definstance " +this.type+ "MsgProp ?" + this.type+ "-var static))";
    		
    	
    	
    	return retString;
    }
    
    public String toConciseString() {
        String retString = new String("(" + this.type + ") ");
        return retString ;
    }
    
    /**
     * Return the receiver of this message
     * 
     * @return Returns the receiver.
     */
    public Role getReceiver() {
        return receiver;
    }
    
    /**
     * Set the receiver of this message
     * 
     * @param receiver The receiver to set.
     */
    public void setReceiver(Role receiver) {
        this.receiver = receiver;
    }
    
    /**
     * Return the sender of this message
     * 
     * @return Returns the sender.
     */
    public Role getSender() {
        return sender;
    }
    
    /**
     * Set the sender for this message
     * 
     * @param sender The sender to set.
     */
    public void setSender(Role sender) {
        this.sender = sender;
    }
    
    /**
     * Return the parameters of this message
     * 
     * @return Returns the parameters.
     */
    public Vector getParameters() {
        return parameters;
    }
    
    /**
     * Set the parameters for this message
     * 
     * @param parameters The parameters to set.
     */
    public void setParameters(Vector slots) {
        this.parameters = slots;
    }
    
    /**
     * Add a parameter to this message
     * 
     * @param s the parameter to add
     */
    public void addSlot(Slot s) {
        if(this.parameters==null) {
            this.parameters=new Vector();
        }
        this.parameters.add(s) ;
    }
    /**
     * @return Returns the type.
     */
    public String getType() {
        return type;
    }
    /**
     * @param type The type to set.
     */
    public void setType(String type) {
        this.type = type;
    }
}
