/*
 * Created on Jun 25, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import java.util.Vector;  
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.* ;
import com.hp.hpl.jena.util.iterator.*;
import edu.ncsu.csc.mas.ontology.exceptions.*;
import edu.ncsu.csc.mas.utils.Constants;

/**
 * An OWL-P protocol.
 * 
 * A protocol has roles and messages. This class creates an in-memory data 
 * structure containing all the roles and messages pertaining to the 
 * protocol in question.
 *  
 *  **NEW VERSION OF PROTOCOL ONTOLOGY HAS RULES IN ADDITION TO ROLES AND
 *  MESSAGES**  Protocol---dictatesRule--->Swrl#Imp
 * 
 * @author Ashok Mallya
 *
 */
public class Protocol extends OwlpObject{
    
    /**
     * The messages used by this protocol
     * 
     * @see edu.ncsu.csc.mas.ontology.Message
     */
    private Vector messages ;
    
    /**
     * The roles that interact using this protocol
     */
    private Vector roles ;
    private String instName; 
    
    /**
     * The rules for this protocol
     * Currently specified using swrl. Each rule has a body and a head, both
     * of which are lists of atoms. THe list is interpreted as a conjunction 
     * of its constituent atoms.
     * 
     * @see edu.ncsu.csc.mas.ontology.Rule
     */
    private Vector rules ;
    
    private KnowledgeBase KB ;
 
	private NiceIterator nIter1, nIter2 ;
	private StmtIterator sIter ;
    
    public Protocol() {
        this.messages = null ;
        this.roles = null ;
    }
    
    public Protocol(OntResource prot)
    throws MalformedConditionException
    {
        if(Constants.DEBUG==true) System.out.println("Constructing a protocol");
        instName =prot.getLocalName();
        for(sIter=prot.listProperties(); sIter.hasNext();) {
            //get the next statement in which this protocol instance is the subject
            Statement protSt = sIter.nextStatement();
           // System.out.println("protStatement: " + protSt.toString());
            if(Constants.VIEW_PROTOCOL_STATEMENTS) System.out.println(protSt) ;
            //get the object of the statement
            RDFNode objRDFNode = protSt.getObject() ;
         //   System.out.println("RDFNode: "+ objRDFNode.toString());
            OntResource protObj ;
            //try and convert this object into an OntResource
            if(objRDFNode.canAs(OntResource.class)) {
                protObj = (OntResource)objRDFNode.as(OntResource.class) ;
            } else 
                continue ;
            //Now see what property related the protocol instance to the object
            if(protSt.getPredicate().getLocalName().equals(Constants.PROTOCOL_ROLE_PROPERTY_NAME)) {
                //the object we found was a role. Create a role for it
                this.addRole(new Role(protObj)) ;
                if(Constants.DEBUG==true) System.out.println("Creating role " + protObj.getLocalName()) ;
            } else if(protSt.getPredicate().getLocalName().equals(Constants.PROTOCOL_MESSAGE_PROPERTY_NAME)) {
                //the object found was a message
                this.addMessage(new Message(protObj)) ;
            } else if(protSt.getPredicate().getLocalName().equals(Constants.PROTOCOL_RULE_PROPERTY_NAME)) {
                //the object found was a rule 
                this.addRule(new Rule(protObj)) ;
                System.out.println( (Rule)this.rules.elementAt(this.rules.size() - 1));
            } else if(protSt.getPredicate().getLocalName().equals(Constants.PROTOCOL_KNOWLEDGEBASE_PROPERTY_NAME)) {
                //the object found was a knowledge base
                this.setKB(new KnowledgeBase(protObj)) ;
            }
        }
    }

    /**
     * @return Returns the messages.
     */
    public Vector getMessages() {
        return this.messages;
    }
    /**
     * @param messages The messages to set.
     */
    public void setMessages(Vector messages) {
        this.messages = messages;
    }
    
    /**
     * Add a message to the set of messages used by theis protocol
     * @param m the message to add
     */
    public void addMessage(Message m){
        if(this.messages==null)
            this.messages = new Vector() ;
        this.messages.add(m) ;
    }
    
    /**
     * @return Returns the roles.
     */
    public Vector getRoles() {
        return this.roles;
    }
    /**
     * @param roles The roles to set.
     */
    public void setRoles(Vector roles) {
        this.roles = roles;
    }
    
    /**
     * Add a role to the set of roles defined by this protocol
     * 
     * @param r the role to add
     */
    public void addRole(Role r) {
        if(this.roles==null)
            this.roles = new Vector() ;
        this.roles.add(r) ;
    }
    
    public void addRule(Rule rul) {
        if(this.rules ==null)
            this.rules = new Vector() ;
        this.rules.add(rul) ;
    }
    
    public String toString() {
        String retString = new String();
        if(this.messages!= null) {
            for(int i=0; i<this.messages.size(); i++) {
                retString += "\n" + ((Message)this.messages.elementAt(i)).toSendMsgJessString(i+1) ;
            }
        }
        return retString ;
    }
    
    /**
     * @return Returns the rules.
     */
    public Vector getRules() {
        return rules;
    }
    /**
     * @param rules The rules to set.
     */
    public void setRules(Vector rules) {
        this.rules = rules;
    }
    
    /**
     * Add a skeleton ruls to a particular role
     *
     */
    public void setSkelRules(Rule rul, Role rol) {
     if(this.roles==null)
         return ;
     for(int i=0; i<this.roles.size(); i++){
         if(((Role)this.roles.elementAt(i)).equals(rol)) {
             ((Role)this.roles.elementAt(i)).addToSkelRules(rul) ;
             break ;
         }
     }
    }
    
    public void setSkelRules(Vector vecRules, Role rol) {
        if(this.roles==null)
            return ;
        for(int i=0; i<this.roles.size(); i++){
            if(((Role)this.roles.elementAt(i)).equals(rol)) {
                ((Role)this.roles.elementAt(i)).setSkelRules(vecRules) ;
                break ;
            }
        }
    }
    
    public Role getRole(Role r) {
        if(this.roles==null)
            return null;
        for(int i=0; i<this.roles.size(); i++) {
            if(((Role)this.roles.elementAt(i)).equals(r)) 
                return (Role)this.getRoles().elementAt(i) ;
        }
        return null ;
    }
    /**
     * @return Returns the kB.
     */
    public KnowledgeBase getKB() {
        return KB;
    }
    /**
     * @param kb The kB to set.
     */
    public void setKB(KnowledgeBase kb) {
        KB = kb;
    }
    public String getInstName()
    {
    	return instName;
    }
}
